import * as path from "node:path";

import { parse } from "json5";
import { DependencyContainer } from "tsyringe";

import { ConfigTypes } from "@spt/models/enums/ConfigTypes";
import { IPostDBLoadMod } from "@spt/models/external/IPostDBLoadMod";
import { IPreSptLoadMod } from "@spt/models/external/IPreSptLoadMod";
import { IInsuranceConfig } from "@spt/models/spt/config/IInsuranceConfig";
import { LogTextColor } from "@spt/models/spt/logging/LogTextColor";
import { IDatabaseTables } from "@spt/models/spt/server/IDatabaseTables";
import { ILogger } from "@spt/models/spt/utils/ILogger";
import { ConfigServer } from "@spt/servers/ConfigServer";
import { DatabaseServer } from "@spt/servers/DatabaseServer";
import { SaveServer } from "@spt/servers/SaveServer";
import { VFS } from "@spt/utils/VFS";

type ModConfig = {
  InstantReturn: {
    enabled: boolean;
    runIntervalSeconds: number;
  };

  PraporMinReturn: number;
  PraporMaxReturn: number;
  PraporMaxStorageTime: number;
  PraporReturnChancePercent: number;

  TherapistMinReturn: number;
  TherapistMaxReturn: number;
  TherapistMaxStorageTime: number;
  TherapistReturnChancePercent: number;
};

class InsuranceTweaks implements IPreSptLoadMod, IPostDBLoadMod {
  public logger: ILogger;
  public tables: IDatabaseTables;
  public modConfig: ModConfig;
  public configServer: ConfigServer;
  public saveServer: SaveServer;

  public preSptLoad(container: DependencyContainer): void {
    this.logger = container.resolve<ILogger>("WinstonLogger");
    const vfs = container.resolve<VFS>("VFS");
    this.modConfig = parse(
      vfs.readFile(path.resolve(__dirname, "../config/config.json5")),
    );
  }

  public postDBLoad(container: DependencyContainer): void {
    this.tables = container
      .resolve<DatabaseServer>("DatabaseServer")
      .getTables();
    this.configServer = container.resolve<ConfigServer>("ConfigServer");
    this.traderTweaks(this.modConfig);
  }

  private traderTweaks(modConfig: ModConfig): void {
    const traders = this.tables.traders;
    const PRAPOR = "54cb50c76803fa8b248b4571";
    const THERAPIST = "54cb57776803fa99248b456e";

    const insuranceConfig = this.configServer.getConfig<IInsuranceConfig>(
      ConfigTypes.INSURANCE,
    );

    if (!modConfig.InstantReturn.enabled) {
      traders[PRAPOR].base.insurance.min_return_hour =
        modConfig.PraporMinReturn;
      traders[PRAPOR].base.insurance.max_return_hour =
        modConfig.PraporMaxReturn;
      traders[THERAPIST].base.insurance.min_return_hour =
        modConfig.TherapistMinReturn;
      traders[THERAPIST].base.insurance.max_return_hour =
        modConfig.TherapistMaxReturn;
    } else {
      insuranceConfig.runIntervalSeconds =
        modConfig.InstantReturn.runIntervalSeconds;
      insuranceConfig.returnTimeOverrideSeconds = 1;
    }

    traders[PRAPOR].base.insurance.max_storage_time =
      modConfig.PraporMaxStorageTime;
    traders[THERAPIST].base.insurance.max_storage_time =
      modConfig.TherapistMaxStorageTime;
    insuranceConfig.returnChancePercent[PRAPOR] =
      modConfig.PraporReturnChancePercent;
    insuranceConfig.returnChancePercent[THERAPIST] =
      modConfig.TherapistReturnChancePercent;

    this.logger.logWithColor(
      "[InsuranceTweaks] Applied insurance settings",
      LogTextColor.GREEN,
    );
  }
}

export const mod = new InsuranceTweaks();
